 LIST	 C=132,n=0,p=16c55,r=dec

;********************************************************
; Project: PIC16C5X as a multiplexed LCD driver.        *
;                                                       *
; Revision history:                                     *
;       04/14/93        original                        *
;********************************************************

; Equates

pic54           equ             0x1ff           ; Define Reset Vectors
pic55           equ             0x1ff
pic56           equ             0x3ff
pic57           equ             0x7ff

rtcc            equ             1               ; f1
pc              equ             2               ; f2
status          equ             3               ; f3
fsr             equ             4               ; f4

porta           equ             5               ; f5
portb           equ             6               ; f6
portc           equ             7               ; f8

; realtime mode registers

currentState    equ             8
msTimer         equ             currentState+1  ; Millisecond timer
sTimerLow       equ             msTimer+1       ; Lower byte second timer
sTimerHigh      equ             sTimerLow+1     ; Upper byte second timer
digit56         equ             sTimerHigh+1
digit34         equ             digit56+1

; Misc definitions

FIVEMSEC        equ             96              ; Assuming 4.096 MHz crystal

w               equ             0
f               equ             1

z               equ             2

; Status register bits

;********************************************************
; Port assignments                                      *
;                                                       *
;  porta - bit0: Common Plane 0                         *
;          bit1: Common Plane 0                         *
;          bit2: Common Plane 1                         *
;          bit3: Common Plane 1                         *
;                                                       *
;  portb - bit0: 6B/DP                                  *
;          bit1: 6C/6D                                  *
;          bit2: 6G/6E                                  *
;          bit3: 6A/6F                                  *
;          bit4: 5B/DP                                  *
;          bit5: 5C/5D                                  *
;          bit6: 5G/5E                                  *
;          bit7: 5A/5F                                  *
;                                                       *
;  portc - bit0: 4B/DP                                  *
;        - bit1: 4C/4D                                  *
;        - bit2: 4G/4E                                  *
;        - bit3: 4A/4F                                  *
;        - bit4: 3B/DP                                  *
;        - bit5: 3C/3D                                  *
;        - bit6: 3G/3E                                  *
;        - bit7: 3A/3F                                  *
;                                                       *
;********************************************************

;********************************************************
;               Macro definitions                       *
;********************************************************
UpdateState     macro   State, Table

        swapf   sTimerLow, w
        andlw   0xf                     ; Isolate digit 5 (offset)
        call    Table
        movwf   digit56
        swapf   digit56, f

        movf    sTimerLow, w
        andlw   0xf                     ; Isolate digit 6 (offset)
        call    Table
        iorwf   digit56, f

        swapf   sTimerHigh, w
        andlw   0xf                     ; Isolate digit 5 (offset)
        call    Table
        movwf   digit34
        swapf   digit34, f

        movf    sTimerHigh, w
        andlw   0xf                     ; Isolate digit 6 (offset)
        call    Table
        iorwf   digit34, f

        movf    digit34, w              ; Display digits 3 & 4
        movwf   portc
        movf    digit56, w
        movwf   portb                   ; Display digits 5 & 6
        endm

        org     0

; Initialize ports A, B, and C and RTCC. In case of output data
; values, set the data latch first, then set the port direction.

Initialize
        movlw   00000001b               ; Set data latch
        movwf   porta
        movlw   00001000b               ; Set I/O direction
        tris    porta

        movlw   00000000b               ; Set levels to low
        movwf   portb
        movlw   00000000b               ; Set as outputs
        tris    portb

        movlw   00000000b               ; Set levels to low
        movwf   portc
        movlw   00000000b               ; Set as outputs
        tris    portc

        movlw   0x04                    ; Set prescaler
        option

        movlw   FIVEMSEC                ; rtcc = 5ms
        movwf   rtcc

        movlw   4
        movwf   currentState

        movlw   0xd
        movwf   msTimer                 ; Initialize millisecond timer

        clrf    sTimerLow               ; Clear second counter
        clrf    sTimerHigh

        retlw   0

; Check timer register for timing out (rtcc = 0). Remain in the
; loop until the timer times out.


; Wait for 5ms timer timeout

Timer_Check
        movf    rtcc, w
        btfss   status, z
        goto    Timer_Check

        movlw   FIVEMSEC
        movwf   rtcc

        decfsz  msTimer, f
        goto    Update_Backplane

        incfsz  sTimerLow                       ; Update second counter
        goto    Update_Backplane
        incf    sTimerHigh, f

; RA0 and RA1 are used to control voltage level for common plane 0.
; RA2 and RA3 are used to control voltage level for common plane 1.
; There are four possible states with different voltage levels as
; follows:
;
; State 0 - cp0 = +5v           ra0=1, ra1=x
;           cp1 = +2.5v         ra2=1, ra3=0
; State 1 - cp0 = +2.5v         ra0=1, ra1=0
;           cp1 = +5v           ra2=1, ra3=x
; State 2 - cp0 =  0v           ra0=0, ra1=x
;           cp1 = +2.5v         ra2=1, ra3=0
; State 3 - cp0 = +2.5v         ra0=1, ra1=0
;           cp1 =  0v           ra2=0, ra3=x


Update_Backplane
        clrwdt                          ; Reset watchdog timer

        decf    currentState, w         ; Update w register
        andlw   0x03                    ; Use only bit0/1
        movwf   currentState            ; Update currentState
        addwf   pc, f

        goto    State3
        goto    State2
        goto    State1
;       goto    State0

; State 0

State0
        UpdateState     State0, S0_Table

        movlw   00000101b
        movwf   porta
        movlw   00000010b
        tris    porta

        retlw   0

S0_Table
        addwf   pc, f                           ; Add offset to pc

        retlw   0100b                           ; 0
        retlw   1100b                           ; 1
        retlw   0010b                           ; 2
        retlw   0000b                           ; 3
        retlw   1000b                           ; 4
        retlw   0001b                           ; 5
        retlw   1111b                           ; 6
        retlw   0100b                           ; 7
        retlw   0000b                           ; 8
        retlw   0000b                           ; 9
        retlw   0000b                           ; a
        retlw   1001b                           ; b
        retlw   1011b                           ; c
        retlw   1000b                           ; d
        retlw   0011b                           ; e
        retlw   0011b                           ; f

; State 1

State1
        UpdateState     State1, S1_Table


        movlw   00000101b
        movwf   porta
        movlw   00001000b
        tris    porta

        retlw   0

S1_Table
        addwf   pc, f

        retlw   0001b                           ; 0
        retlw   1111b                           ; 1
        retlw   1001b                           ; 2
        retlw   1101b                           ; 3
        retlw   0111b                           ; 4
        retlw   0101b                           ; 5
        retlw   1111b                           ; 6
        retlw   1111b                           ; 7
        retlw   0001b                           ; 8
        retlw   0101b                           ; 9
        retlw   0011b                           ; a
        retlw   0001b                           ; b
        retlw   1001b                           ; c
        retlw   1001b                           ; d
        retlw   0001b                           ; e
        retlw   0011b                           ; f

; State 2

State2
        UpdateState     State2, S2_Table

        movlw   00000100b
        movwf   porta
        movlw   00000010b
        tris    porta

        retlw   0

S2_Table
        addwf   pc, f

        retlw   1011b                           ; 0
        retlw   0011b                           ; 1
        retlw   1101b                           ; 2
        retlw   1111b                           ; 3
        retlw   0111b                           ; 4
        retlw   1110b                           ; 5
        retlw   1110b                           ; 6
        retlw   1011b                           ; 7
        retlw   1111b                           ; 8
        retlw   1111b                           ; 9
        retlw   1111b                           ; a
        retlw   0110b                           ; b
        retlw   0100b                           ; c
        retlw   0111b                           ; d
        retlw   1100b                           ; e
        retlw   1100b                           ; f

; State 3

State3
        UpdateState     State3, S3_Table

        movlw   00000001b
        movwf   porta
        movlw   00001000b
        tris    porta

        retlw   0

S3_Table
        addwf   pc, f

        retlw   1110b                           ; 0
        retlw   0000b                           ; 1
        retlw   0110b                           ; 2
        retlw   0010b                           ; 3
        retlw   1000b                           ; 4
        retlw   1010b                           ; 5
        retlw   1110b                           ; 6
        retlw   0000b                           ; 7
        retlw   1110b                           ; 8
        retlw   1010b                           ; 9
        retlw   1100b                           ; a
        retlw   1110b                           ; b
        retlw   0110b                           ; c
        retlw   0110b                           ; d
        retlw   1110b                           ; e
        retlw   1100b                           ; f

; Main code

Start
        call    Initialize
Repeat
        call    Timer_Check
        goto    Repeat

        org     pic55

System_Reset
        goto    Start

        END
